<?php
/**
**/

defined('_JEXEC') or die;

use Joomla\CMS\Factory;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\PluginHelper;
use Joomla\CMS\Http\HttpFactory;
use Joomla\CMS\Version;
use Joomla\CMS\Extension\ExtensionHelper;
use Joomla\Registry\Registry;


class plgInstallerBongovo extends CMSPlugin
{
  private const URL = 'https://www.bongovo.cz/download/translationlist.xml';
	private const USER_AGENT = 'Bongovo.cz Joomla Installer plugin';

	/**
	 * The loaded extensions.
	 *
	 * @var    array
	 * @since  4.0.0
	 */
	private static $loadedExtensions = [];

	public static function getExtensionRecord(string $element, string $type, ?int $clientId = null, ?string $folder = null): ?\stdClass
	{
		if (Version::MAJOR_VERSION > 3)
			return ExtensionHelper::getExtensionRecord($element, $type, $clientId, $folder);

		if ($type === 'plugin' && $folder === null)
		{
			throw new \InvalidArgumentException(sprintf('`$folder` is required when `$type` is `plugin` in %s()', __METHOD__));
		}

		if (\in_array($type, ['module', 'language', 'template'], true) && $clientId === null)
		{
			throw new \InvalidArgumentException(
				sprintf('`$clientId` is required when `$type` is `module`, `language` or `template` in %s()', __METHOD__)
			);
		}

		$key = $element . '.' . $type . '.' . $clientId . '.' . $folder;

		if (!\array_key_exists($key, self::$loadedExtensions))
		{
			$db = Factory::getDbo();
			$query = $db->getQuery(true)
				->select('*')
				->from($db->quoteName('#__extensions'))
				->where(
					[
						$db->quoteName('element') . ' = ' . $db->quote($element),
						$db->quoteName('type') . ' = '  . $db->quote($type),
					]
				);

			if ($clientId !== null)
				$query->where($db->quoteName('client_id') . ' = ' . $clientId);

			if ($folder !== null)
				$query->where($db->quoteName('folder') . ' = ' . $db->quote($folder));

			$query->setLimit(1);
			$db->setQuery($query);

			self::$loadedExtensions[$key] = $db->loadObject();
		}

		return self::$loadedExtensions[$key];
	}

	public function loadTranslations(): ?array
	{
		$http = HttpFactory::getHttp();
		$http->setOption('User-Agent', self::USER_AGENT);
		$response = $http->get(self::URL);

		if ($response->code !== 200)
			return null;

		if (!$response->body)
			return null;

		$translations = [];

		try
		{
			$xml = new \SimpleXMLElement($response->body);
		} catch (\Exception $exception) {
			return null;
		}

		$version = new Version;
		foreach ($xml->extension as $extension)
		{
			$translation = current($extension->attributes());
			$translation = array_merge([
				'name' => null,
				'extensiontype' => null,
				'extensionfolder' => null,
				'extensionclient' => null,
				'extensionelement' => null,
				'extensionminversion' => null,
				'extensionmaxversion' => null,
				'language' => null,
				'element' => null,
				'type' => 'file',
				'version' => null,
				'description' => null,
				'downloadurl' => null,
				'joomlaminversion' => null,
				'joomlamaxversion' => null,

				'extensionversion' => null,
				'installedversion' => null,
			], $translation);

			$translation = (object)$translation;

			if ($translation->joomlaminversion) {
				if (!version_compare($version->getShortVersion(), $translation->joomlaminversion, '>='))
					continue;
			}

			if ($translation->joomlamaxversion) {
				if (!version_compare($version->getShortVersion(), $translation->joomlamaxversion, '<='))
					continue;
			}

			if ($translation->element && $translation->type) {
				$extension = $this->getExtensionRecord($translation->element, $translation->type);
				if ($extension) {
					$manifest                      = new Registry($extension->manifest_cache);
					$translation->installedversion = $manifest->get('version');
				}
			}

			if ($translation->extensionelement && $translation->extensiontype) {
				$extension = $this->getExtensionRecord($translation->extensionelement, $translation->extensiontype, $translation->extensionclient == 'site' ? 0 : 1, $translation->extensionfolder);
				if ($extension) {
					$manifest                      = new Registry($extension->manifest_cache);
					$translation->extensionversion = $manifest->get('version');
				}
			}

			$translations[] = $translation;
		}

		return $translations;
	}

	public function onInstallerAddInstallationTab()
	{
		// Load language files
		$this->loadLanguage();

		$tab            = array();
		$tab['name']    = 'bongovo';
		$tab['label']   = Text::_('PLG_INSTALLER_BONGOVO_TEXT');

		// Render the input
		ob_start();
		include PluginHelper::getLayoutPath('installer', 'bongovo');
		$tab['content'] = ob_get_clean();

		return $tab;
	}
}
